@@ -1,5 +1,5 @@
 /*
- * Copyright (C), 2000-2003 by the monit project group.
+ * Copyright (C), 2000-2004 by the monit project group.
  * All Rights Reserved.
  *
  * This program is free software; you can redistribute it and/or
@@ -53,10 +53,10 @@
  *  Implementation of the alert module
  *
  *  @author Jan-Henrik Haukeland, <hauk@tildeslash.com>
- *  @author Martin Pala <martin.pala@iol.cz>
+ *  @author Martin Pala <martinp@tildeslash.com>
  *  @author Christian Hopp <chopp@iei.tu-clausthal.de>
  *
- *  @version \$Id: alert.c,v 1.29 2003/10/25 19:17:10 hauk Exp $
+ *  @version \$Id: alert.c,v 1.33 2004/02/18 22:31:42 chopp Exp $
  *  @file
  */
 
@@ -84,16 +84,17 @@ void handle_alert(Event_T E) {
 
   s= Event_get_source(E);
 
-  if(s->maillist) {
+  if(s->maillist||Run.maillist) {
 
     Mail_T m;
+    Mail_T n;
     sigset_t ns, os;
     Mail_T list= NULL;
 
     set_signal_block(&ns, &os);
     
     /*
-     * Build a mail-list with recipients that has registered interest
+     * Build a mail-list with local recipients that has registered interest
      * for this event.
      */
     for(m= s->maillist; m; m= m->next) {
@@ -125,6 +126,49 @@ void handle_alert(Event_T E) {
 
     }
 
+    /*
+     * Build a mail-list with global recipients that has registered interest
+     * for this event. Recipients which are defined in the service localy
+     * overrides the same recipient events which are registered globaly.
+     */
+    for(m= Run.maillist; m; m= m->next) {
+      
+      int skip= FALSE;
+
+      for(n= s->maillist; n; n= n->next) {
+        if(IS(m->to, n->to)) {
+	  skip= TRUE;
+	  break;
+	}
+      }
+
+      if(!skip &&
+           (IS_EVENT_SET(m->events, Event_get_id(E)) ||
+	   (Event_get_id(E) == EVENT_FAILED))
+	) {
+
+	Mail_T tmp;
+
+	NEW(tmp);
+	
+	copy_mail(tmp, m);
+
+	if(Event_get_message(E))
+	  tmp->opt_message= xstrdup(Event_get_message(E));
+
+	substitute(&tmp, s->name, EVENT_DESCRIPTION(E));
+	
+	replace_bare_linefeed(&tmp);
+	
+	tmp->next= list;
+	list= tmp;
+	
+	DEBUG("%s notification is sent to %s\n", EVENT_DESCRIPTION(E), m->to);
+
+      }
+
+    }
+
     if(list) {
       
       sendmail(list);
@@ -147,7 +191,7 @@ static void substitute(Mail_T *m, char *name, const char *event) {
   ASSERT(m && name && event);
 
   if(gethostname(host, sizeof(host)) < 0) {
-    snprintf(host, STRLEN, LOCALHOST);
+    snprintf(host, STRLEN, "%s", LOCALHOST);
   }
 
   replace_string(&(*m)->from, "$HOST", host);
