@@ -339,6 +339,9 @@ static void do_start(Service_T s) {
     if(s->type==TYPE_PROCESS) {
       /* We only wait for a process type, other service types does not
        * have a pid file to watch */
+      LOCK(Run.mutex)
+        Run.wait_start++;
+      END_LOCK;
       status= pthread_create(&thread, NULL, wait_start, s);
       if(status != 0) {
 	log("Warning: Failed to create the start controller thread. "
@@ -551,6 +554,10 @@ static void *wait_start(void *service) {
     Event_post(s, EVENT_FAILED, "Failed to start '%s'\n", s->name);
   }
 
+  LOCK(Run.mutex)
+    Run.wait_start--;
+  END_LOCK;
+
   return NULL;
   
 }
