@@ -1,5 +1,5 @@
 /*
- * Copyright (C), 2000-2003 by the monit project group.
+ * Copyright (C), 2000-2004 by the monit project group.
  * All Rights Reserved.
  *
  * This program is free software; you can redistribute it and/or
@@ -62,21 +62,25 @@
 #include "base64.h"
 #include "event.h" 
 #include "alert.h"
-#include "monit_process.h"
-#include "monit_device.h"
+#include "process.h"
+#include "device.h"
 
 #define ACTION(c) !strncasecmp(req->url, c, sizeof(c))
 
-
 /* URL Commands supported */
 #define HOME        "/"
 #define STOP        "/_stop"
 #define TEST        "/_monit"
 #define ABOUT       "/_about"
 #define PIXEL       "/_pixel"
+#define STATUS      "/_status"
 #define RUN         "/_runtime"
 #define VIEWLOG     "/_viewlog"
 
+/* Status output formats */
+#define XML_STATUS   0
+#define TEXT_STATUS  1
+
 /* Private prototypes */
 static int is_readonly(HttpRequest);
 static void printPixel(HttpResponse);
@@ -93,7 +97,6 @@ static void not_found(HttpRequest, HttpResponse);
 static void do_runtime(HttpRequest, HttpResponse);
 static void do_viewlog(HttpRequest, HttpResponse);
 static void handle_action(HttpRequest, HttpResponse);
-static void print_status(Service_T, HttpResponse res);
 static void is_monit_running(HttpRequest, HttpResponse);
 static void do_service(HttpRequest, HttpResponse, char *);
 static void do_service_device(HttpRequest, HttpResponse, Service_T);
@@ -101,9 +104,27 @@ static void do_service_directory(HttpRequest, HttpResponse, Service_T);
 static void do_service_file(HttpRequest, HttpResponse, Service_T);
 static void do_service_process(HttpRequest, HttpResponse, Service_T);
 static void do_service_host(HttpRequest, HttpResponse, Service_T);
-static void print_alerts(HttpResponse, Service_T);
+static void print_alerts(HttpResponse, Mail_T);
 static void print_buttons(HttpRequest, HttpResponse, Service_T);
 static void print_service_common_params(HttpResponse, Service_T);
+static void print_status(HttpResponse, int);
+static void status_start(HttpResponse, int);
+static void status_stop(HttpResponse, int);
+static void status_service_xml(Service_T, HttpResponse);
+static void status_service_text(Service_T, HttpResponse);
+static int get_service_status(Service_T);
+
+static  struct {
+  const char *type;
+  const char *up;
+  const char *down;
+} status_text[]= {
+  {"Device", "accessible", "not accessible or has errors"},
+  {"Directory", "accessible", "not accessible or has errors"},
+  {"File", "accessible", "not accessible or has errors"},
+  {"Process", "running", "not running or has errors"},
+  {"Host", "online with all services", "not online or service errors"}
+};
 
 
 /**
@@ -113,10 +134,10 @@ static void print_service_common_params(HttpResponse, Service_T);
  *  monit.
  *
  *  @author Jan-Henrik Haukeland, <hauk@tildeslash.com>
- *  @author Martin Pala <martin.pala@iol.cz>
+ *  @author Martin Pala <martinp@tildeslash.com>
  *  @author Christian Hopp <chopp@iei.tu-clausthal.de>
  *
- *  @version \$Id: cervlet.c,v 1.111 2003/10/25 19:17:13 hauk Exp $
+ *  @version \$Id: cervlet.c,v 1.126 2004/02/29 22:24:45 martinp Exp $
  *
  *  @file
  */
@@ -183,6 +204,14 @@ static void doGet(HttpRequest req, HttpResponse res) {
     }
   } else if(ACTION(PIXEL)) {
     printPixel(res);
+  } else if(ACTION(STATUS)) {
+    const char *format= get_parameter(req, "format");
+    if(format && starts_with(format, "xml"))
+      print_status(res, XML_STATUS);
+    else {
+      /* Text status output is default */
+      print_status(res, TEXT_STATUS);
+    }
   } else {
     handle_action(req, res);
   }
@@ -251,18 +280,11 @@ static void do_about(HttpRequest req, HttpResponse res) {
     "monit " VERSION "</font></a></center><br><br>");
   out_print(res,
     "<ul>"
-    "<li>Copyright &copy; 2000-2003 by <a "
+    "<li>Copyright &copy; 2000-2004 by <a "
     "href=\"http://www.tildeslash.com/monit/who.html\">the monit "
     "project group</a>. All Rights Reserved.<br> </small><p>"
     "<li>Portions of this software are copyright &copy; 1995, 1996 "
-    "<a href='http://www.gnu.org/'>Free Software Foundation, Inc.</a></ul>"
-    "<p><br>Please consider making a "
-    "<a class=grey href=\"http://www.tildeslash.com/monit/index.html#support\">"
-    "donation</a> and support our continued work with monit. We are also "
-    "looking for hardware donation so we can develop and test monit on many "
-    "different systems. If you would like to donate hardware, please contact "
-    "us at <a href=\"mailto:monitgroup@tildeslash.com\">"
-    "monitgroup@tildeslash.com</a><p>");
+    "<a href='http://www.gnu.org/'>Free Software Foundation, Inc.</a></ul>");
   out_print(res,
    "<table cellspacing=\"0\" cellpadding=\"0\" width=\"100%%\" border=\"0\">"
    " <tr bgcolor=\"#6F6F6F\">"
@@ -277,7 +299,10 @@ static void do_about(HttpRequest req, HttpResponse res) {
     "WITHOUT ANY WARRANTY; without even the implied warranty of "
     "MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the "
     "<a href='http://www.tildeslash.com/gpl.html'>GNU "
-    "General Public License</a> for more details.");
+    "General Public License</a> for more details."
+    "<p><br><center><p><i>Please consider making a "
+    "<a class=grey href=\"http://www.tildeslash.com/monit/index.html#support\">"
+    "donation</a> and support our continued work with monit.</i></center>");
 
 }
 
@@ -340,6 +365,7 @@ static void do_runtime(HttpRequest req, HttpResponse res) {
     out_print(res,
 	    "<tr><td>Default mail message</td><td>%s</td></tr>",
             Run.MailFormat.message);
+
   out_print(res,
 	    "<tr><td>Poll time</td><td>%d seconds</td></tr>",
 	    Run.polltime);
@@ -348,6 +374,9 @@ static void do_runtime(HttpRequest req, HttpResponse res) {
 	    Run.bind_addr?Run.bind_addr:"Any/All");
   out_print(res,
 	    "<tr><td>httpd portnumber</td><td>%d</td></tr>", Run.httpdport);
+  out_print(res,
+	    "<tr><td>httpd signature</td><td>%d</td></tr>",
+	    Run.httpdsig?"True":"False");
   out_print(res,
 	    "<tr><td>Use ssl encryption</td><td>%s</td></tr>",
 	    Run.httpdssl?"True":"False");
@@ -380,6 +409,9 @@ static void do_runtime(HttpRequest req, HttpResponse res) {
 	    (Run.credentials!=NULL)?"Basic Authentication":
 	    has_hosts_allow()?"Host allow list":
 	    "No authentication");
+
+  print_alerts(res, Run.maillist);
+
   out_print(res,"</table>");
 
   if(!is_readonly(req)) {
@@ -473,7 +505,7 @@ static void do_viewlog(HttpRequest req, HttpResponse res) {
 static void handle_action(HttpRequest req, HttpResponse res) {
 
   char *name= req->url;
-  char *action= get_parameter(req, "action");
+  const char *action= get_parameter(req, "action");
   
   if(exist_service(++name)) {
     if(action) {
@@ -552,9 +584,6 @@ static void handle_action(HttpRequest req, HttpResponse res) {
 	  LOCK(s->mutex)
 	    check_service(name, action);
 	  END_LOCK;
-      } else if(IS(action, "status")) {
-	print_status(s, res);
-	goto quit;
       }
       
     }
@@ -570,7 +599,6 @@ static void handle_action(HttpRequest req, HttpResponse res) {
   }
 
   quit:
-  FREE(action);
   reset_depend();
   
 }
@@ -614,207 +642,7 @@ static void do_service(HttpRequest req, HttpResponse res, char *name) {
 }
 
 
-static void print_status(Service_T s, HttpResponse res) {
 
-  pid_t  pid= -1;
-  Socket_T S= res->S;
-  struct stat stat_buf;
-  
-  res->is_committed= TRUE;
-  
-  switch(s->type) {
-    
-  case TYPE_PROCESS:
-      
-      if((pid= is_process_running(s))) {
-	
-	ProcInfo_T pi= s->procinfo;
-	char *uptime= get_process_uptime(s->path, " ");
-	
-	if(Run.doprocess) {
-	  
-	  socket_print(S,
-		       "Process '%s' is running with pid [%d]\n\tUptime: %s "
-		       "CPU: %.1f%% Memory w/o children: %.1f%% [%ldkB]\n"
-		       "\tChildren: %i Memory w/ children: %.1f%% [%ldkB]\n"
-		       "\tMonitoring status: %s\n",
-		       s->name, (int)pid, uptime, pi->cpu_percent/10.0,
-		       pi->mem_percent/10.0,pi->mem_kbyte,
-		       pi->children,pi->total_mem_percent /
-		       10.0,pi->total_mem_kbyte,
-		       statusnames[s->do_monitor]);
-	  
-	} else {
-	  
-	  socket_print(S,
-		       "Process '%s' is running with pid [%d]\n\tUptime: %s "
-		       "\tMonitoring status: %s\n",
-		       s->name, (int)pid, uptime,statusnames[s->do_monitor]);
-	  
-	}
-	
-	FREE(uptime);
-	
-      } else {
-	
-	socket_print(S, "Process '%s' is not running\n",  s->name);
-	  
-      }
-      break;
-      
-  case TYPE_DEVICE:
-      if(DeviceInfo_Usage(s->devinfo, s->path) &&
-	 stat(s->path, &stat_buf) == 0 ) {
-	
-        if(s->devinfo->f_files > 0) {
-	  
-          socket_print(S,
-		       "Device '%s' %s\n\t"
-		       "Space: %.1f%% [%.1f MB] Inodes: %.1f%% [%ld objects]\n\t"
-		       "UID: %d GID: %d Permission: %o\n\t"
-		       "Monitoring status: %s\n",
-		       s->name,
-		       (s->perm&&s->perm->has_error)?
-		       "permission error":
-		       (s->uid&&s->uid->has_error)?
-		       "UID error":
-		       (s->gid&&s->gid->has_error)?
-		       "GID error":
-		       "is accessible",
-		       (float) 100 *
-		       (s->devinfo->f_blocks - s->devinfo->f_blocksfreetotal) /
-		       s->devinfo->f_blocks,
-		       (float) (s->devinfo->f_blocks -
-				s->devinfo->f_blocksfreetotal) /
-		       1048576 * s->devinfo->f_bsize,
-		       (float) 100 * (s->devinfo->f_files -
-				      s->devinfo->f_filesfree) /
-		       s->devinfo->f_files,
-		       s->devinfo->f_files - s->devinfo->f_filesfree,
-		       (int)stat_buf.st_uid, (int)stat_buf.st_gid,
-		       (int)(stat_buf.st_mode & 07777),
-		       statusnames[s->do_monitor]);
-        } else {
-          socket_print(S,
-		       "Device '%s' %s\n\t"
-		       "Space: %.1f%% [%.1f MB]\n\t"
-		       "UID: %d GID: %d Permission: %o\n\t"
-		       "Monitoring status: %s\n",
-		       s->name,
-		       (s->perm&&s->perm->has_error)?
-		       "permission error":
-		       (s->uid&&s->uid->has_error)?
-		       "UID error":
-		       (s->gid&&s->gid->has_error)?
-		       "GID error":
-		       "is accessible",
-		       (float) 100 *
-		       (s->devinfo->f_blocks - s->devinfo->f_blocksfreetotal) /
-		       s->devinfo->f_blocks,
-		       (float) (s->devinfo->f_blocks -
-				s->devinfo->f_blocksfreetotal) /
-		       1048576 * s->devinfo->f_bsize,
-		       (int)stat_buf.st_uid, (int)stat_buf.st_gid,
-		       (int)(stat_buf.st_mode & 07777),
-		       statusnames[s->do_monitor]);
-        }
-      } else {
-        socket_print(S,
-		     "Device '%s' is not accessible\n\t"
-		     "Space: - Inodes: -\n\t"
-		     "UID: - GID: - Permission: -\n\t"
-		     "Monitoring status: %s\n",
-		     s->name, statusnames[s->do_monitor]);
-	
-      }
-      break;
-      
-  case TYPE_FILE:
-      if( (stat(s->path, &stat_buf) == 0) && S_ISREG(stat_buf.st_mode)) {
-	
-        socket_print(S,
-		     "File '%s' %s\n\t"
-		     "Size: %luB UID: %d GID: %d Permission: %o\n\t"
-		     "Monitoring status: %s\n",
-		     s->name,
-		     (s->perm&&s->perm->has_error)?
-		     "permission error":
-		     (s->uid&&s->uid->has_error)?
-		     "UID error":
-		     (s->gid&&s->gid->has_error)?
-		     "GID error":
-		     "exist",
-		     (unsigned long)stat_buf.st_size, (int)stat_buf.st_uid,
-		     (int)stat_buf.st_gid, (int)(stat_buf.st_mode & 07777),
-		     statusnames[s->do_monitor]);
-      } else {
-
-	socket_print(S, "File '%s' doesn't exist\n\tMonitoring status: %s\n",
-		     s->name, statusnames[s->do_monitor]);
-      }
-      break;
-      
-  case TYPE_DIRECTORY:
-      if( (stat(s->path, &stat_buf) == 0) && S_ISDIR(stat_buf.st_mode)) {
-
-	socket_print(S,
-		     "Directory '%s' %s\n\t"
-		     "UID: %d GID: %d Permission: %o\n\t"
-		     "Monitoring status: %s\n",
-		     s->name,
-		     (s->perm&&s->perm->has_error)?
-		     "permission error":
-		     (s->uid&&s->uid->has_error)?
-		     "UID error":
-		     (s->gid&&s->gid->has_error)?
-		     "GID error":
-		     "exist",
-		     (int)stat_buf.st_uid,
-		     (int)stat_buf.st_gid, (int)(stat_buf.st_mode & 07777),
-		     statusnames[s->do_monitor]);
-	
-      } else {
-
-	socket_print(S,
-		     "Directory '%s' doesn't exist\n\tMonitoring status: %s\n",
-		     s->name, statusnames[s->do_monitor]);
-      }
-      break;
-      
-  case TYPE_REMOTE:
-      if(s->portlist) {
-	Port_T port;
-	int available= TRUE;
-	
-	for(port= s->portlist; port; port= port->next) {
-	  if(!port->is_available) {
-	    available= FALSE;
-	    break;
-	  }
-	}
-	
-	if(!available) {
-	  
-	  socket_print(S,
-		       "Host '%s'\n\tPort %d is not available\n"
-		       "\tMonitoring status: %s\n",
-		       s->name, port->port, statusnames[s->do_monitor]);
-	  
-	} else {
-
-	  socket_print(S,
-		       "Host '%s'\n\tAll monitored ports are available\n"
-		       "\tMonitoring status: %s\n",
-		       s->name, statusnames[s->do_monitor]);
-	}
-      }
-      break;
-      
-  default:
-      break;
-  }
-
-}
 
 
 static void printPixel(HttpResponse res) {
@@ -905,6 +733,7 @@ static void do_home_process(HttpRequest req, HttpResponse res) {
 		  "<td align=\"right\">%.1f%% [%ld&nbsp;kB]</td></tr>",
 		  on?"bgcolor=\"#EFEFEF\"":"",
 		  s->name, s->name,
+		  !s->do_monitor?"<font color='#ff8800'>not monitored</font>":
 		  isrunning?
 		  "<font color='#00ff00'>running</font>":
 		  "<font color='#ff0000'>not running</font>",
@@ -922,6 +751,7 @@ static void do_home_process(HttpRequest req, HttpResponse res) {
 		  "<td align=\"right\">- [-]</td></tr>",
 		  on?"bgcolor=\"#EFEFEF\"":"",
 		  s->name, s->name,
+		  !s->do_monitor?"<font color='#ff8800'>not monitored</font>":
 		  "<font color=red>not running</font>"
 		  );
       }
@@ -933,6 +763,7 @@ static void do_home_process(HttpRequest req, HttpResponse res) {
 		"</tr>",
 		on?"bgcolor=\"#EFEFEF\"":"",
 		s->name, s->name,
+		!s->do_monitor?"<font color='#ff8800'>not monitored</font>":
 		isrunning?
 		"<font color=green>running</font>":
 		"<font color=red>not running</font>",
@@ -980,12 +811,14 @@ static void do_home_device(HttpRequest req, HttpResponse res) {
       out_print(res,
 		"<tr %s>"
 		"<td width=\"20%\"><a href='/%s'>%s</a></td>"
-		"<td align=\"left\"><font color=red>not accessible</font></td>"
+		"<td align=\"left\">%s</td>"
 		"<td align=\"right\">- [-]</td>"
 		"<td align=\"right\">- [-]</td>"
 		"</tr>",
 		on?"bgcolor=\"#EFEFEF\"":"",
-		s->name, s->name);
+		s->name, s->name,
+		!s->do_monitor?"<font color='#ff8800'>not monitored</font>":
+		"<font color=red>not accessible</font>");
     } else {
       
       if(s->devinfo->f_files > 0) {
@@ -1003,6 +836,7 @@ static void do_home_device(HttpRequest req, HttpResponse res) {
 		  "<font color='#ff0000'>Permission Error</font>":
 		  (s->uid&&s->uid->has_error)?
 		  "<font color='#ff0000'>UID Error</font>":
+		  !s->do_monitor?"<font color='#ff8800'>not monitored</font>":
 		  (s->gid&&s->gid->has_error)?
 		  "<font color='#ff0000'>GID Error</font>":
 		  "<font color='#00ff00'>accessible</font>",
@@ -1029,6 +863,7 @@ static void do_home_device(HttpRequest req, HttpResponse res) {
 		  "<font color='#ff0000'>Permission Error</font>":
 		  (s->uid&&s->uid->has_error)?
 		  "<font color='#ff0000'>UID Error</font>":
+		  !s->do_monitor?"<font color='#ff8800'>not monitored</font>":
 		  (s->gid&&s->gid->has_error)?
 		  "<font color='#ff0000'>GID Error</font>":
 		  "<font color='#00ff00'>accessible</font>",
@@ -1085,15 +920,16 @@ static void do_home_file(HttpRequest req, HttpResponse res) {
       out_print(res,
 		"<tr %s>"
 		"<td width=\"20%\"><a href='/%s'>%s</a></td>"
-		"<td align=\"left\"><font color=red>doesn't exist</font></td>"
+		"<td align=\"left\">%s</td>"
 		"<td align=\"right\">-</td>"
 		"<td align=\"right\">-</td>"
 		"<td align=\"right\">-</td>"
 		"<td align=\"right\">-</td>"
 		"</tr>",
 		on?"bgcolor=\"#EFEFEF\"":"",
-		s->name, s->name
-		);
+		s->name, s->name,
+		!s->do_monitor?"<font color='#ff8800'>not monitored</font>":
+		"<font color=red>doesn't exist</font>");
 
     } else {
       
@@ -1112,6 +948,7 @@ static void do_home_file(HttpRequest req, HttpResponse res) {
 		"<font color='#ff0000'>Permission Error</font>":
 		(s->uid&&s->uid->has_error)?
 		"<font color='#ff0000'>UID Error</font>":
+		!s->do_monitor?"<font color='#ff8800'>not monitored</font>":
 		(s->gid&&s->gid->has_error)?
 		"<font color='#ff0000'>GID Error</font>":
 		"<font color='#00ff00'>accessible</font>",
@@ -1166,14 +1003,15 @@ static void do_home_directory(HttpRequest req, HttpResponse res) {
       out_print(res,
 		"<tr %s>"
 		"<td width=\"20%\"><a href='/%s'>%s</a></td>"
-		"<td align=\"left\"><font color=red>doesn't exist</font></td>"
+		"<td align=\"left\">%s</td>"
 		"<td align=\"right\">-</td>"
 		"<td align=\"right\">-</td>"
 		"<td align=\"right\">-</td>"
 		"</tr>",
 		on?"bgcolor=\"#EFEFEF\"":"",
-		s->name, s->name
-		);
+		s->name, s->name,
+		!s->do_monitor?"<font color='#ff8800'>not monitored</font>":
+		"<font color=red>doesn't exist</font>");
       
     } else {
       
@@ -1191,6 +1029,7 @@ static void do_home_directory(HttpRequest req, HttpResponse res) {
 		"<font color='#ff0000'>Permission Error</font>":
 		(s->uid&&s->uid->has_error)?
 		"<font color='#ff0000'>UID Error</font>":
+		!s->do_monitor?"<font color='#ff8800'>not monitored</font>":
 		(s->gid&&s->gid->has_error)?
 		"<font color='#ff0000'>GID Error</font>":
 		"<font color='#00ff00'>accessible</font>",
@@ -1253,9 +1092,10 @@ static void do_home_remote(HttpRequest req, HttpResponse res) {
       }
       
       if(available) {
-	out_print(res, "<td align=\"left\">"
-		  "<font color='#00ff00'>available</font></td>"
-		  "<td align=\"right\">");
+	out_print(res, "<td align=\"left\">%s</td>"
+		  "<td align=\"right\">",
+		  !s->do_monitor?"<font color='#ff8800'>not monitored</font>":
+		  "<font color='#00ff00'>available</font>");
 	for(port= s->portlist; port; port= port->next) {
 	  if(port != s->portlist)
 	      out_print(res, "&nbsp;&nbsp;<b>|</b>&nbsp;&nbsp;");
@@ -1263,9 +1103,10 @@ static void do_home_remote(HttpRequest req, HttpResponse res) {
 	}
 	out_print(res, "</td>");
       } else {
-	out_print(res, "<td align=\"left\">"
-		  "<font color='#ff0000'>not available</font></td>"
+	out_print(res, "<td align=\"left\">%s</td>"
 		  "<td align=\"right\">[%s] at port %d</td>",
+		  !s->do_monitor?"<font color='#ff8800'>not monitored</font>":
+		  "<font color='#ff0000'>not available</font>",
 		  port->protocol->name, port->port);
       }
     } else if(s->icmplist) {
@@ -1441,7 +1282,7 @@ static void do_service_device(HttpRequest req, HttpResponse res, Service_T s) {
     }
   }
   
-  print_alerts(res, s);
+  print_alerts(res, s->maillist);
   
   out_print(res, "</table>");
   
@@ -1530,7 +1371,7 @@ static void do_service_directory(HttpRequest req,
 	    "<font>",
 	    (int)stat_buf.st_gid);
   
-  print_alerts(res, s);
+  print_alerts(res, s->maillist);
   
   out_print(res, "</table>");
   
@@ -1602,10 +1443,27 @@ static void do_service_file(HttpRequest req, HttpResponse res, Service_T s) {
       }
     }
   }
-  if(s->checksum)
-      out_print(res, "<tr><td>Associated checksum</td><td>"
-		"if failed %s then %s</td></tr>",
-		s->checksum->md5, actionnames[s->checksum->action]);
+
+  if(s->checksum) {
+    switch (s->checksum->type) {
+    case HASH_MD5:
+        out_print(res, "<tr><td>Associated checksum</td><td>"
+                  "if failed MD5(%s) then %s</td></tr>",
+                  s->checksum->hash, actionnames[s->checksum->action]);
+        break;
+    case HASH_SHA1:
+        out_print(res, "<tr><td>Associated checksum</td><td>"
+                  "if failed SHA1(%s) then %s</td></tr>",
+                  s->checksum->hash, actionnames[s->checksum->action]);
+        break;
+    default:
+        out_print(res, "<tr><td>Associated checksum</td><td>"
+                  "if failed UNKNOWN(%s) then %s</td></tr>",
+                  s->checksum->hash, actionnames[s->checksum->action]);
+        break;
+    }
+  }
+  
   if(s->perm)
       out_print(res, "<tr><td>Associated permission</td><td>"
 		"if failed %o then %s</td></tr>",
@@ -1640,7 +1498,7 @@ static void do_service_file(HttpRequest req, HttpResponse res, Service_T s) {
 	    "<font>",
 	    (int)stat_buf.st_gid);
 
-  print_alerts(res, s);
+  print_alerts(res, s->maillist);
 
   out_print(res, "</table>");
 
@@ -1807,7 +1665,7 @@ static void do_service_process(HttpRequest req, HttpResponse res, Service_T s) {
     }
   }
   
-  print_alerts(res, s);
+  print_alerts(res, s->maillist);
   
   out_print(res, "</table>");
   
@@ -1890,7 +1748,7 @@ static void do_service_host(HttpRequest req, HttpResponse res, Service_T s) {
 		icmpnames[n->type], n->timeout, actionnames[n->action]);
   }
   
-  print_alerts(res, s);
+  print_alerts(res, s->maillist);
   
   out_print(res, "</table>");
 
@@ -1902,11 +1760,11 @@ static void do_service_host(HttpRequest req, HttpResponse res, Service_T s) {
 /* ------------------------------------------------------------------------- */
 
 
-static void print_alerts(HttpResponse res, Service_T s) {
+static void print_alerts(HttpResponse res, Mail_T s) {
 
   Mail_T r;
   
-  for(r= s->maillist; r; r= r->next) {
+  for(r= s; r; r= r->next) {
     out_print(res,
 	      "<tr bgcolor=\"#EFEFEF\"><td>Alert mail to</td>"
 	      "<td>%s</td></tr>", r->to?r->to:"");
@@ -2062,3 +1920,149 @@ static int is_readonly(HttpRequest req) {
   return FALSE;
 
 }
+
+
+/* ----------------------------------------------------------- Status output */
+
+
+static void print_status(HttpResponse res, int format) {
+
+  Service_T s;
+
+  set_content_type(res, "text/plain");
+  status_start(res, format);
+  for(s= servicelist_conf; s; s= s->next_conf) {
+    if(format==XML_STATUS) {
+      status_service_xml(s, res);
+    } else {
+      /* Text status is default */
+      status_service_text(s, res);
+    }
+  }
+  status_stop(res, format);
+
+}
+
+
+static void status_start(HttpResponse res, int format) {
+  
+  char *uptime= get_process_uptime(Run.pidfile, " ");
+  
+  if(format==XML_STATUS) {
+    out_print(res, "<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>\r\n");
+    out_print(res, "<monit>\r\n\t<server>\r\n\t\t<version>%s</version>"
+	      "\r\n\t\t<uptime>%s</uptime>\r\n\t</server>\r\n", 
+	      VERSION, trim(uptime));
+  } else {
+    /* Text status is default */
+    out_print(res, "The monit daemon uptime: %s\n\n", uptime);
+  }
+
+  FREE(uptime);
+  
+}
+
+
+static void status_stop(HttpResponse res, int format) {
+ 
+  if(format==XML_STATUS)
+    out_print(res, "</monit>\r\n");
+  
+}
+
+
+static void status_service_xml(Service_T s, HttpResponse res) {
+  
+  out_print(res, "\t<service>\r\n\t\t<type>%s</type>\r\n\t\t<name>%s</name>"
+	    "\r\n\t\t<status>%d</status>\r\n\t\t<monitored>%d</monitored>"
+	    "\r\n\t</service>\r\n",
+	    status_text[s->type].type, 
+	    s->name, get_service_status(s), s->do_monitor);
+  
+}
+
+
+static void status_service_text(Service_T s, HttpResponse res) {
+  
+  char prefix[STRLEN];
+  
+  snprintf(prefix, STRLEN, "%s '%s'", status_text[s->type].type, s->name);
+  out_print(res, "%-35s %s\n", prefix,
+	    !s->do_monitor?"not monitored":
+	    get_service_status(s)?status_text[s->type].up:
+	    status_text[s->type].down);
+  
+}
+
+
+static int get_service_status(Service_T s) {
+
+  struct stat stat_buf;
+  
+  switch(s->type) {
+  case TYPE_PROCESS:
+    return (is_process_running(s)>0);
+    break;
+  case TYPE_DEVICE:
+    if(DeviceInfo_Usage(s->devinfo, s->path) && 
+       stat(s->path, &stat_buf) == 0) {
+      if(s->uid&&s->uid->has_error)
+	return FALSE;
+      else if(s->gid&&s->gid->has_error)
+	return FALSE;
+      else
+	return TRUE;
+    } else {
+      return FALSE;
+    }
+    break;
+  case TYPE_FILE:
+      if((stat(s->path, &stat_buf) == 0) && S_ISREG(stat_buf.st_mode)) {
+	if(s->perm&&s->perm->has_error)
+	  return FALSE;
+	else if(s->uid&&s->uid->has_error)
+	  return FALSE;
+	else if(s->gid&&s->gid->has_error)
+	  return FALSE;
+	else 
+	  return TRUE;
+      } else {
+	return FALSE;
+      }
+      break;
+  case TYPE_DIRECTORY:
+    if((stat(s->path, &stat_buf) == 0) && S_ISDIR(stat_buf.st_mode)) {
+      if(s->perm&&s->perm->has_error)
+	return FALSE;
+      else if(s->uid&&s->uid->has_error)
+	return FALSE;
+      else if(s->gid&&s->gid->has_error)
+	return FALSE;
+      else
+	return TRUE;
+    } else {
+      return FALSE;
+    }
+    break;
+  case TYPE_REMOTE:
+    if(s->portlist) {
+	Port_T port;
+	int available= TRUE;
+	for(port= s->portlist; port; port= port->next) {
+	  if(!port->is_available) {
+	    available= FALSE;
+	    break;
+	  }
+	}
+	return available;
+    } else {
+      return FALSE;
+    }
+    break;
+  default:
+    break;
+  }
+  
+  return FALSE;
+
+}
