@@ -101,7 +101,7 @@
  *  @author Jan-Henrik Haukeland, <hauk@tildeslash.com>
  *  @author Christian Hopp, <chopp@iei.tu-clausthal.de>
  *
- *  @version \$Id: processor.c,v 1.39 2004/02/18 22:31:42 chopp Exp $
+ *  @version \$Id: processor.c,v 1.40 2004/04/04 08:08:11 hauk Exp $
  *
  *  @file
  */
@@ -139,21 +139,16 @@ static int get_next_token(char *s, int *cursor, char **r);
  * @param s A Socket_T representing the client connection
  */
 void *http_processor(void *s) {
-  
   Socket_T S= s;
-
+  
   if(! can_read(socket_get_socket(S), REQUEST_TIMEOUT)) {
     internal_error(S, SC_REQUEST_TIMEOUT, "Time out when handling the Request");
     goto shutdown;
   }
-  
   do_service(S);
-
-  shutdown:
+ shutdown:
   socket_free(&S);
-
   return NULL;
-  
 }
 
 
@@ -163,10 +158,8 @@ void *http_processor(void *s) {
  * @param doPostFunc doPost function 
  */
 void add_Impl(void *doGetFunc, void *doPostFunc) {
-
   Impl.doGet= doGetFunc;
   Impl.doPost= doPostFunc;
-
 }
 
 
@@ -177,7 +170,6 @@ void add_Impl(void *doGetFunc, void *doPostFunc) {
  * @param msg Optional error message (may be NULL)
  */
 void send_error(HttpResponse res, int code, const char *msg) {
-
   char server[STRLEN];
   const char *err= get_status_string(code);
 
@@ -190,7 +182,6 @@ void send_error(HttpResponse res, int code, const char *msg) {
 	   "<hr><a href='%s'><font size=-1>%s</font></a>"\
 	   "</body></html>\r\n",
 	    code, err, err, msg?msg:"", SERVER_URL, get_server(server, STRLEN));
-  
 }
 
 
@@ -201,11 +192,9 @@ void send_error(HttpResponse res, int code, const char *msg) {
  * @param location An absolute url to redirect to
  */
 void send_redirect(HttpResponse res, const char *location) {
-
   reset_response(res);
   set_status(res, SC_MOVED_TEMPORARILY);
   set_header(res, "Location", location);
-
 }
 
 
@@ -224,9 +213,7 @@ void send_redirect(HttpResponse res, const char *location) {
  * @param m A formated string to be sent to the client
  */
 void out_print(HttpResponse res, const char *m, ...) {
-
   if(m) {
-    
     char *buf;
     va_list ap;
     long need= 0;
@@ -235,31 +222,20 @@ void out_print(HttpResponse res, const char *m, ...) {
     va_start(ap, m);
     buf= format(m, ap, &need);
     va_end(ap);
-
     have= res->bufsize - res->bufused;
-      
     if(have <= need) {
-      
       res->outputbuffer=
-	  xresize(res->outputbuffer,(res->bufsize + need + RES_STRLEN));
-      
+	xresize(res->outputbuffer,(res->bufsize + need + RES_STRLEN));
       res->bufsize+= (need + RES_STRLEN);
-      
       if(!res->bufused) {
-	
-	  memset(res->outputbuffer, 0, res->bufsize);
-
+	memset(res->outputbuffer, 0, res->bufsize);
       }
-      
     }
-
     memcpy(&res->outputbuffer[res->bufused], buf, need); 
     res->bufused+= need;
     res->outputbuffer[res->bufused]= 0; 
     FREE(buf);
-    
   }
-  
 }
 
 
@@ -274,16 +250,13 @@ void out_print(HttpResponse res, const char *m, ...) {
  * @param value Header key value
  */
 void set_header(HttpResponse res, const char *name, const char *value) {
-
   HttpHeader h;
 
   ASSERT(res);
   ASSERT(name);
-
   NEW(h);
   h->name= xstrdup(name);
   h->value= xstrdup(value);
-  
   if(res->headers) {
     HttpHeader n, p;
     for( n= p= res->headers; p; n= p, p= p->next) {
@@ -294,15 +267,10 @@ void set_header(HttpResponse res, const char *name, const char *value) {
 	return;
       }
     }
-    
     n->next= h;
-    
   } else {
-    
     res->headers= h;
-    
   }
-
 }
 
 
@@ -313,10 +281,8 @@ void set_header(HttpResponse res, const char *name, const char *value) {
  * @param msg The status code string message 
  */
 void set_status(HttpResponse res, int code) {
-  
   res->status= code;
   res->status_msg= get_status_string(code);
-  
 }
 
 
@@ -326,9 +292,7 @@ void set_status(HttpResponse res, int code) {
  * @param mime Mime content type, e.g. text/html
  */
 void set_content_type(HttpResponse res, const char *mime) {
-
   set_header(res, "Content-Type", mime);
-
 }
 
 
@@ -339,21 +303,14 @@ void set_content_type(HttpResponse res, const char *mime) {
  * @return The value of the specified header, NULL if not found
  */
 const char *get_header(HttpRequest req, const char *name) {
-
   HttpHeader p;
 
   for(p= req->headers; p; p= p->next) {
-    
     if(!strcasecmp(p->name, name)) {
-      
-	return (p->value);
-
+      return (p->value);
     }
-    
   }
-
   return NULL;
-
 }
 
 
@@ -364,21 +321,14 @@ const char *get_header(HttpRequest req, const char *name) {
  * @return The value of the specified parameter, or NULL if not found
  */
 const char *get_parameter(HttpRequest req, const char *name) {
-
   HttpParameter p;
 
   for(p= req->params; p; p= p->next) {
-    
     if(!strcasecmp(p->name, name)) {
-      
-	return (p->value);
-
+      return (p->value);
     }
-    
   }
-
   return NULL;
-
 }
 
 
@@ -390,21 +340,15 @@ const char *get_parameter(HttpRequest req, const char *name) {
  * @return A String containing all headers set in the Response object
  */
 char *get_headers(HttpResponse res) {
-
   HttpHeader p;
   char buf[RES_STRLEN];
   char *b= buf;
 
   *buf=0;
-  
   for(p= res->headers; (((b-buf) + STRLEN) < RES_STRLEN) && p; p= p->next) {
-
     b+= snprintf(b, STRLEN,"%s: %s\r\n", p->name, p->value);
-    
   }
-  
   return buf[0]?xstrdup(buf):NULL;
-
 }
 
 
@@ -416,7 +360,6 @@ char *get_headers(HttpResponse res) {
  * code.
  */
 const char *get_status_string(int status) {
-
   switch (status) {
   case SC_OK:
       return "OK";
@@ -500,7 +443,6 @@ const char *get_status_string(int status) {
       return "Unknown HTTP status";
     } 
   }
-
 }
 
 
@@ -518,24 +460,15 @@ static void do_service(Socket_T s) {
   if(res && req) {
     if(is_authenticated(req, res)) {
       if(IS(req->method, METHOD_GET)) {
-
 	Impl.doGet(req, res);
-
       } else if(IS(req->method, METHOD_POST)) {
-
 	Impl.doPost(req, res);
-
       } else {
-	
 	send_error(res, SC_NOT_IMPLEMENTED, "Method not implemented");
-
       }
     }
-
     send_response(res);
-
   }
-
   done(req, res);
 }
 
@@ -544,19 +477,13 @@ static void do_service(Socket_T s) {
  * Return a (RFC1123) Date string
  */
 static char *get_date(char *result, int size) {
-
   time_t now;
   
   time(&now);
-  
   if(strftime(result, size, DATEFMT, gmtime(&now)) <= 0) {
-    
     *result= 0;
-
   }
-  
   return result;
-  
 }
 
 
@@ -564,11 +491,8 @@ static char *get_date(char *result, int size) {
  * Return this server name + version
  */
 static char *get_server(char *result, int size) {
-
   snprintf(result, size, "%s %s", SERVER_NAME, Run.httpdsig?SERVER_VERSION:"");
-  
   return result;
-  
 }
 
 
@@ -577,18 +501,15 @@ static char *get_server(char *result, int size) {
  * commited, this function does nothing.
  */
 static void send_response(HttpResponse res) {
-
   Socket_T S= res->S;
 
   if(!res->is_committed) {
-    
     char date[STRLEN];
     char server[STRLEN];
     char *headers= get_headers(res);
 
     get_date(date, STRLEN);
     get_server(server, STRLEN);
-    
     socket_print(S, "%s %d %s\r\n", res->protocol, res->status,
 		 res->status_msg);
     socket_print(S, "Date: %s\r\n", date);
@@ -598,14 +519,10 @@ static void send_response(HttpResponse res) {
     if(headers)
 	socket_print(S, "%s", headers);
     socket_print(S, "\r\n");
-      
     if(res->bufused)
 	socket_write(S, res->outputbuffer, res->bufused);
-
     FREE(headers);
-    
   }
-
 }
 
 
@@ -616,52 +533,39 @@ static void send_response(HttpResponse res) {
  * Returns a new HttpRequest object wrapping the client request
  */
 static HttpRequest create_HttpRequest(Socket_T S) {
-
   HttpRequest req;
   char url[REQ_STRLEN];
   char line[REQ_STRLEN];
   char protocol[STRLEN]; 
   char method[REQ_STRLEN];
 
-    
   if(socket_readln(S, line, REQ_STRLEN) == NULL) {
     internal_error(S, SC_BAD_REQUEST, "No request found");
     return NULL;
   }
-
   chomp(line,REQ_STRLEN);
-
   /* RATS: ignore */ /* chomp does zero termination */
   if(sscanf(line, "%s %s HTTP/%3[1.0]", method, url, protocol) != 3) {
     internal_error(S, SC_BAD_REQUEST, "Cannot parse request");
     return NULL;
   }
-
   if(strlen(url) >= MAX_URL_LENGTH) {
     internal_error(S, SC_BAD_REQUEST, "[error] URL too long");
     return NULL;
   }
-
   NEW(req);
   req->S= S;
   url_decode(url);
   req->url= xstrdup(url);
   req->method= xstrdup(method);
   req->protocol= xstrdup(protocol); 
-
   create_headers(req);
-
   if(!create_parameters(req)) {
-    
     internal_error(S, SC_BAD_REQUEST, "Cannot parse Request parameters");
     destroy_HttpRequest(req);
-
     return NULL;
-    
   }
-    
   return req;
-  
 }
 
 
@@ -670,7 +574,6 @@ static HttpRequest create_HttpRequest(Socket_T S) {
  * the set_XXX methods to change the object.
  */
 static HttpResponse create_HttpResponse(Socket_T S) {
-  
   HttpResponse res;
 
   NEW(res);
@@ -682,9 +585,7 @@ static HttpResponse create_HttpResponse(Socket_T S) {
   res->is_committed= FALSE;
   res->protocol= SERVER_PROTOCOL;
   res->status_msg= get_status_string(SC_OK);
-  
   return res;
-  
 }
 
 
@@ -692,7 +593,6 @@ static HttpResponse create_HttpResponse(Socket_T S) {
  * Create HTTP headers for the given request
  */
 static void create_headers(HttpRequest req) {
-
   Socket_T S;
   char *value;
   HttpHeader p;
@@ -700,17 +600,12 @@ static void create_headers(HttpRequest req) {
   char line[REQ_STRLEN];
 
   S= req->S;
-  
   while(1) {
-
     if(! socket_readln(S, line, sizeof(line)))
 	break;
-
     if(IS(line, "\r\n") || IS(line, "\n"))
 	break;
-    
     if( NULL != (value= strchr(line, ':'))) {
-      
       NEW(header);
       *value++= '\0';
       trim(line);
@@ -719,23 +614,15 @@ static void create_headers(HttpRequest req) {
       chomp(value, (REQ_STRLEN-(value-line)));
       header->name= xstrdup(line);
       header->value= xstrdup(value);
-      
       if(req->headers) {
-	
 	for(p= req->headers; p->next; p= p->next)
-	    /* Empty */;
+	  /* Empty */;
 	p->next= header;
-	
       } else {
-	
 	req->headers= header;
-	
       }
-      
     }
-    
   }
-  
 }
 
 
@@ -744,7 +631,6 @@ static void create_headers(HttpRequest req) {
  * occurs.
  */
 static int create_parameters(HttpRequest req) {
-
   char query_string[REQ_STRLEN]= {0};
 
   if(IS(req->method, METHOD_POST) && get_header(req, "Content-Length")) {
@@ -754,7 +640,7 @@ static int create_parameters(HttpRequest req) {
     if(1 != sscanf(get_header(req, "Content-Length"), "%d", &len)) {
       return FALSE;
     }
-    if(len < 0 || len > REQ_STRLEN)
+    if(len < 0 || len >= REQ_STRLEN)
       return FALSE;
     if(len==0)
       return TRUE;
@@ -777,9 +663,7 @@ static int create_parameters(HttpRequest req) {
     }
     req->params= parse_parameters(query_string);
   }
-  
   return TRUE;
-  
 }
 
 
@@ -790,12 +674,11 @@ static int create_parameters(HttpRequest req) {
  * Clear the response output buffer and headers
  */
 static void reset_response(HttpResponse res) {
-
-  if(res->headers)destroy_entry(res->headers);
+  if(res->headers)
+    destroy_entry(res->headers);
   memset(res->outputbuffer, 0, res->bufsize);
   res->bufused= 0;
   res->headers= NULL; /* Release Pragma */
-  
 }
 
 
@@ -803,10 +686,8 @@ static void reset_response(HttpResponse res) {
  * Finalize the request and response object.
  */
 static void done(HttpRequest req, HttpResponse res) {
-
   destroy_HttpRequest(req);
   destroy_HttpResponse(res);
-
 }
 
 
@@ -814,20 +695,18 @@ static void done(HttpRequest req, HttpResponse res) {
  * Free a HttpRequest object
  */
 static void destroy_HttpRequest(HttpRequest req) {
-
   if(req) {
-    
     FREE(req->method);
     FREE(req->url); 
     FREE(req->pathinfo);
     FREE(req->protocol);
     FREE(req->remote_user);
-    if(req->headers)destroy_entry(req->headers);
-    if(req->params)destroy_entry(req->params);
+    if(req->headers)
+      destroy_entry(req->headers);
+    if(req->params)
+      destroy_entry(req->params);
     FREE(req);
-    
   }
-  
 }
 
 
@@ -835,15 +714,12 @@ static void destroy_HttpRequest(HttpRequest req) {
  * Free a HttpResponse object
  */
 static void destroy_HttpResponse(HttpResponse res) {
-
   if(res) {
-    
     FREE(res->outputbuffer);
-    if(res->headers) destroy_entry(res->headers);
+    if(res->headers) 
+      destroy_entry(res->headers);
     FREE(res);
-    
   }
-  
 }
 
 
@@ -852,19 +728,14 @@ static void destroy_HttpResponse(HttpResponse res) {
  * HttpParameter are of this type.
  */
 static void destroy_entry(void *p) {
-  
   struct entry *h= p; 
   
   if(h->next) {
-    
     destroy_entry(h->next);
-    
   }
-
   FREE(h->name);
   FREE(h->value);
   FREE(h);
- 
 }
 
 
@@ -875,26 +746,18 @@ static void destroy_entry(void *p) {
  * Do Basic Authentication if this auth. style is allowed. 
  */
 static int is_authenticated(HttpRequest req, HttpResponse res) {
-
   if(Run.credentials!=NULL) {
-
     if(! basic_authenticate(req)) {
-      
       send_error(res, SC_UNAUTHORIZED,
 		 "You are <b>not</b> authorized to access <i>monit</i>. "
 		 "Either you supplied the wrong credentials (e.g. bad "
 		 "password), or your browser doesn't understand how to supply "
 		 "the credentials required");
       set_header(res, "WWW-Authenticate", "Basic realm=\"monit\"");
-      
       return FALSE;
-      
     }
-
   }
-
   return TRUE;
-
 }
 
 
@@ -903,60 +766,42 @@ static int is_authenticated(HttpRequest req, HttpResponse res) {
  * the user.
  */
 static int basic_authenticate(HttpRequest req) {
-
-  int rv= FALSE;
+  int n;
+  char *password;
+  char buf[STRLEN];
+  char uname[STRLEN];
   const char *credentials= get_header(req, "Authorization");
 
-  if(credentials) {
-
-    unsigned char *cr= xcalloc(sizeof(unsigned char), strlen(credentials)+1);
-
-    if(decode_base64(cr, strchr(credentials, ' '))) {
-
-      char *passwd;
-      char uname[STRLEN];
-      long unamelen=(long) strchr(cr, ':') - (long) cr;
-
-      /* Extract the username the user */
-      strncpy(uname, cr, unamelen);
-      uname[unamelen]='\0';
-
-      /* Check if user 'uname' does exist */
-      
-      if(NULL==get_user_credentials(uname)) {
-
-	log("Warning: Client '%s' supplied unknown user '%s'"
-	    " accessing monit httpd\n", socket_get_remote_host(req->S),
-            uname); 
-
-      } else {
-      
-        passwd=strchr(cr,':')+1;
-        /* Check if 'uname' has supplied the right passwd */
-
-        if(compare_user_credentials(uname,  passwd)) {
-
-	  req->remote_user= xstrdup(uname);
-          rv= TRUE;
-          
-        } else {
-          
-          log("Warning: Client '%s' supplied wrong password for user '%s'"
-              " accessing monit httpd\n", socket_get_remote_host(req->S),
-              uname); 
-          
-        }
-
-      }
-      
-    }
-    
-    FREE(cr);
-
+  if(! (credentials && starts_with(credentials, "Basic "))) {
+    return FALSE;
   }
-  
-  return rv;
-  
+  strncpy(buf, &credentials[6], STRLEN);
+  if((n= decode_base64((unsigned char*)uname, buf))<=0) {
+    return FALSE;
+  }
+  uname[n]= 0;
+  password= strchr(uname, ':');
+  if(password==NULL) {
+    return FALSE;
+  }
+  *password++= 0;
+  if(*uname==0 || *password==0) {
+    return FALSE;
+  }
+  /* Check if user 'uname' exist */
+  if(NULL==get_user_credentials(uname)) {
+    log("Warning: Client '%s' supplied unknown user '%s'"
+	" accessing monit httpd\n", socket_get_remote_host(req->S), uname); 
+    return FALSE;
+  }
+  /* Check if 'uname' has supplied the right passwd */
+  if(! compare_user_credentials(uname,  password)) {
+    log("Warning: Client '%s' supplied wrong password for user '%s'"
+	" accessing monit httpd\n", socket_get_remote_host(req->S), uname); 
+    return FALSE;
+  }
+  req->remote_user= xstrdup(uname);
+  return TRUE;
 }
 
 
@@ -969,14 +814,12 @@ static int basic_authenticate(HttpRequest req) {
  * properly; i.e. with a valid HttpRequest and a valid HttpResponse.
  */
 static void internal_error(Socket_T S, int status, char *msg) {
-  
   char date[STRLEN];
   char server[STRLEN];
   const char *status_msg= get_status_string(status);
   
   get_date(date, STRLEN);
   get_server(server, STRLEN);
-  
   socket_print(S, 
 	       "%s %d %s\r\n"
 	       "Date: %s\r\n"
@@ -990,7 +833,6 @@ static void internal_error(Socket_T S, int status, char *msg) {
 	       "</body></html>\r\n",
 	       SERVER_PROTOCOL, status, status_msg, date, server,
 	       status_msg, status_msg, msg, SERVER_URL, server);
-  
 }
 
 
@@ -999,10 +841,8 @@ static void internal_error(Socket_T S, int status, char *msg) {
  * linked list of HttpParameters
  */
 static HttpParameter parse_parameters(char *query_string) {
-
 #define KEY 1
 #define VALUE 2
-
   int token;
   int cursor= 0;
   char *key= NULL;
@@ -1023,16 +863,12 @@ static HttpParameter parse_parameters(char *query_string) {
       key= NULL;
     }
   }
-
   return head;
-  
- error:
+error:
   FREE(key);
   FREE(value);
   destroy_entry(head);
-  
   return NULL;
-  
 }
 
 
@@ -1040,9 +876,8 @@ static HttpParameter parse_parameters(char *query_string) {
  * A mini-scanner for tokenizing a query string
  */
 static int get_next_token(char *s, int *cursor, char **r) {
-  
   int i= *cursor;
-    
+
   while(s[*cursor]) {
     if(s[*cursor+1]=='=') {
       *cursor+= 1;
@@ -1060,7 +895,5 @@ static int get_next_token(char *s, int *cursor, char **r) {
     }
     *cursor+= 1;
   }
-
   return FALSE;
-
 }
