@@ -262,6 +262,10 @@ static void do_reinit() {
   char *bind_addr;
   int port= Run.httpdport;
 
+  /* wait for all wait_start threads to finish */
+  while(Run.wait_start)
+    sleep(1);
+
   Run.doreload= FALSE;
   
   bind_addr= Run.bind_addr?xstrdup(Run.bind_addr):NULL;
@@ -446,6 +450,11 @@ static void do_exit() {
     log("%s daemon with pid [%d] killed\n", prog, (int)getpid());
 
   }
+
+  /* wait for all wait_start threads to finish */
+  while(Run.wait_start)
+    sleep(1);
+
   gc();
   exit(0);
  
