@@ -1,5 +1,5 @@
 /*
- * Copyright (C), 2000-2003 by the monit project group.
+ * Copyright (C), 2000-2004 by the monit project group.
  * All Rights Reserved.
  *
  * This program is free software; you can redistribute it and/or
@@ -71,8 +71,9 @@
 #endif
 
 
+#include "monitor.h"
 #include "process.h"
-#include "sysdep.h"
+#include "process_sysdep.h"
 
 /**
  *  System dependent resource gathering code for AIX
@@ -81,7 +82,7 @@
  *  @author Jan-Henrik Haukeland, <hauk@tildeslash.com>
  *  @author Christian Hopp <chopp@iei.tu-clausthal.de>
  *
- *  @version \$Id: sysdep_AIX.c,v 1.3 2003/10/25 19:17:13 hauk Exp $
+ *  @version \$Id: sysdep_AIX.c,v 1.7 2004/02/29 22:24:45 martinp Exp $
  *
  *  @file
  * most of the code is based on monitor-2.1.5 available on various sources
@@ -278,18 +279,17 @@ int initprocesstree_sysdep(ProcessTree_T ** reference) {
   /* Allocate the tree */
   
   pt = xcalloc(sizeof(ProcessTree_T), treesize);
-  memset( pt, 0, sizeof(ProcessTree_T)*treesize );
 
   /* Insert data from /proc directory */
 
   for(i=0;i<treesize;i++) {
 
-    pt[i]->pid=procs[i].pi_pid;
-    pt[i]->ppid=procs[i].pi_ppid;
+    pt[i].pid=procs[i].pi_pid;
+    pt[i].ppid=procs[i].pi_ppid;
 
     if(procs[i].pi_stat==SZOMB) {
 
-      pt[i]->mem_kbyte= 0;
+      pt[i].mem_kbyte= 0;
 
     } else {
 
@@ -298,12 +298,13 @@ int initprocesstree_sysdep(ProcessTree_T ** reference) {
 
       if(rv== -1) {
 
-	pt[i]->mem_kbyte= 0;
+	pt[i].mem_kbyte= 0;
+
+      } else {
+
+        pt[i].mem_kbyte = (user.ui_drss+user.ui_trss)*4;
 
       }
-
-      p->mem_kbyte = (user.ui_drss+user.ui_trss)*4;
-      
     }    
   }
 
