@@ -1,5 +1,5 @@
 /*
- * Copyright (C), 2000-2003 by the monit project group.
+ * Copyright (C), 2000-2004 by the monit project group.
  * All Rights Reserved.
  *
  * This program is free software; you can redistribute it and/or
@@ -80,8 +80,9 @@
 #include <nlist.h>
 #endif
 
+#include "monitor.h"
 #include "process.h"
-#include "sysdep.h"
+#include "process_sysdep.h"
 
 #define NPROCS 10000
 static struct pst_status psall[NPROCS];
@@ -94,7 +95,7 @@ static int page_size;
  *  @author Jan-Henrik Haukeland, <hauk@tildeslash.com>
  *  @author Christian Hopp <chopp@iei.tu-clausthal.de>
  *  @author Joe Bryant <JBryant@RiteAid.com>
- *  @version \$Id: sysdep_HPUX.c,v 1.5 2003/10/25 19:17:13 hauk Exp $
+ *  @version \$Id: sysdep_HPUX.c,v 1.8 2004/02/29 22:24:45 martinp Exp $
  *
  *  @file
  */
@@ -245,7 +246,6 @@ int initprocesstree_sysdep(ProcessTree_T ** reference) {
   /* Allocate the tree */
 
   pt = xcalloc(sizeof(ProcessTree_T), treesize);
-  memset( pt, 0, sizeof(ProcessTree_T)*globbuf.gl_pathc );
 
   /* Inspect data */
 
