@@ -1,5 +1,5 @@
 /*
- * Copyright (C), 2000-2003 by the monit project group.
+ * Copyright (C), 2000-2004 by the monit project group.
  * All Rights Reserved.
  *
  * This program is free software; you can redistribute it and/or
@@ -70,18 +70,19 @@
 #include <glob.h>
 #endif
 
+#include "monitor.h"
 #include "process.h"
-#include "sysdep.h"
+#include "process_sysdep.h"
 
 /**
  *  System dependent resource gathering code for Linux.
  *
  *  @author Jan-Henrik Haukeland, <hauk@tildeslash.com>
  *  @author Christian Hopp <chopp@iei.tu-clausthal.de>
- *  @author Martin Pala <martin.pala@iol.cz>
+ *  @author Martin Pala <martinp@tildeslash.com>
  *  @author Arkadiusz Miskiewicz <arekm@pld-linux.org>
  *
- *  @version \$Id: sysdep_LINUX.c,v 1.8 2003/10/25 19:17:13 hauk Exp $
+ *  @version \$Id: sysdep_LINUX.c,v 1.12 2004/02/29 22:24:45 martinp Exp $
  *
  *  @file
  */
@@ -254,7 +255,6 @@ int initprocesstree_sysdep(ProcessTree_T ** reference) {
   /* Allocate the tree */
 
   pt = xcalloc(sizeof(ProcessTree_T), treesize);
-  memset( pt, 0, sizeof(ProcessTree_T)*globbuf.gl_pathc );
 
   /* Insert data from /proc directory */
 
