@@ -1,5 +1,5 @@
 /*
- * Copyright (C), 2000-2003 by the monit project group.
+ * Copyright (C), 2000-2004 by the monit project group.
  * All Rights Reserved.
  *
  * This program is free software; you can redistribute it and/or
@@ -39,7 +39,7 @@
  *
  *  @author Jan-Henrik Haukeland, <hauk@tildeslash.com>
  *
- *  @version \$Id: nntp.c,v 1.12 2003/10/25 19:17:14 hauk Exp $
+ *  @version \$Id: nntp.c,v 1.14 2004/02/18 22:31:42 chopp Exp $
  *
  *  @file
  */
@@ -56,8 +56,9 @@ int check_nntp(Socket_T s) {
     return FALSE;
   }
 
-  chomp(buf);
+  chomp(buf, STRLEN);
   
+  /* RATS: ignore */ /* chomp does zero termination */
   sscanf(buf, "%d %s", &status, msg);
   if(status != 200) {
     log("NNTP error: %s\n", buf);
@@ -74,8 +75,9 @@ int check_nntp(Socket_T s) {
     return FALSE;
   }
 
-  chomp(buf);
+  chomp(buf, STRLEN);
   
+  /* RATS: ignore */ /* chomp does zero termination */
   sscanf(buf, "%d %s", &status, msg);
   if(status != 205) {
     log("NNTP error: %s\n", buf);
