@@ -1,5 +1,5 @@
 /*
- * Copyright (C), 2000-2003 by the monit project group.
+ * Copyright (C), 2000-2004 by the monit project group.
  * All Rights Reserved.
  *
  * This program is free software; you can redistribute it and/or
@@ -63,7 +63,7 @@
  *  file, if not set, the default is ~/.monit.state.
  *
  *  @author Jan-Henrik Haukeland, <hauk@tildeslash.com>
- *  @version \$Id: state.c,v 1.15 2003/10/25 19:17:11 hauk Exp $
+ *  @version \$Id: state.c,v 1.18 2004/02/14 22:26:43 hauk Exp $
  *  @file
  */
 
@@ -72,7 +72,7 @@
 
 
 /**
- * Fields from the Service_T object type, which we are interessted in
+ * Fields from the Service_T object type, which we are interested in
  * when handling the state.
  */
 typedef struct mystate {
@@ -229,16 +229,6 @@ void state_update() {
 }
 
 
-/**
- * Finalize state
- */
-void state_finalize() {
-  
-  unlink(Run.statefile);
-
-}
-
-
 /* ----------------------------------------------------------------- Private */
 
 
