@@ -1,5 +1,5 @@
 /*
- * Copyright (C), 2000-2003 by the monit project group.
+ * Copyright (C), 2000-2004 by the monit project group.
  * All Rights Reserved.
  *
  * This program is free software; you can redistribute it and/or
@@ -89,12 +89,14 @@
 #include "monitor.h"
 #include "engine.h"
 #include "md5.h"
+#include "sha.h"
 #include "base64.h"
 #include "alert.h"
-#include "monit_process.h"
+#include "process.h"
 
 
 /* Private prototypes */
+static char x2c(char *hex);
 static char *is_str_defined(char *);
 static int is_unsafe(unsigned char *c);
 
@@ -103,11 +105,11 @@ static int is_unsafe(unsigned char *c);
  *  General purpose utility methods.
  *
  *  @author Jan-Henrik Haukeland, <hauk@tildeslash.com>
- *  @author "Martin Pala" <martin.pala@iol.cz>
+ *  @author "Martin Pala" <martinp@tildeslash.com>
  *  @author Christian Hopp <chopp@iei.tu-clausthal.de>
  *  @author Michael Amster, <mamster@webeasy.com> 
  *
- *  @version \$Id: util.c,v 1.102 2003/10/25 19:17:11 hauk Exp $
+ *  @version \$Id: util.c,v 1.115 2004/02/29 22:24:44 martinp Exp $
  *  @file
  */
 
@@ -147,14 +149,21 @@ char *stripfilename(char* path) {
  * Removes everything from the first newline (CR|LF)
  * @param string A string to be chomped
  */
-void chomp(char *string) {
+void chomp(char *string, int len) {
   
-  char *p;
+  char *p=string;
+  int   i;
 
   ASSERT(string);
 
-  if((p= strchr(string, '\r')) || (p= strchr(string, '\n'))) *p= 0;
+  for (i=0;i<len;i++,p++) {
+    if((*p== '\r') || (*p=='\n')) {
+      *p= 0;
+      return;
+    }
+  }
   
+  string[len-1]=0;
 }
 
 
@@ -176,7 +185,7 @@ char *trim(char *s) {
 
 
 /**
- * Remove leading white space [ \t\r\n] from the string. 
+ * Remove leading white space [ \t\r\n] from the string.
  * @param s A string
  * @return s with leading spaces removed
  */
@@ -199,18 +208,17 @@ char *ltrim(char *s) {
  * @return s with trailing spaces removed
  */
 char *rtrim(char *s) {
-  
+
   char *t= s;
 
   ASSERT(s);
 
-  while(*s) s++; 
+  while(*s) s++;
   while(*--s==' ' || *s=='\t' || *s=='\r' || *s=='\n') *s= '\0';
 
   return t;
-  
-}
 
+}
 
 /**
  * Remove any enclosing quotes ["'] from the string
@@ -250,6 +258,25 @@ void trim_quotes(char *s) {
 }
 
 
+/**
+ * Replace all occurrences of the <code>old</code> char in the string
+ * <code>s</code> with the <code>new</code> char.
+ * @param s A string
+ * @param old The old char
+ * @param new The new char
+ * @return s where all occurrence of old are replaced with new
+ */
+char *replace_char(char *s, char old, char new) {
+
+  char *t= s;
+
+  while (s&&*s) { if(*s==old) *s=new; s++; }
+
+  return (t);
+
+}
+
+
 /**
  * Replace all occurrences of the sub-string old in the string src
  * with the sub-string new. The method is case sensitive for the
@@ -334,7 +361,7 @@ int count_words(char *s, const char *word) {
  * @param b The sub-string to test a against
  * @return TRUE if a starts with b, otherwise FALSE
  */
-int starts_with(char *a, char *b) {
+int starts_with(const char *a, const char *b) {
 
   if((!a || !b) || *a!=*b) return FALSE;
 
@@ -412,7 +439,7 @@ void handle_string_escapes(char *buf) {
  * @param name A service name as stated in the config file
  * @return the named service or NULL if not found
  */
-Service_T get_service(char *name) {
+Service_T get_service(const char *name) {
 
   Service_T s;
 
@@ -434,7 +461,7 @@ Service_T get_service(char *name) {
  * @return TRUE if the service name exist in the
  * servicelist, otherwise FALSE
  */
-int exist_service(char *name) {
+int exist_service(const char *name) {
 
   Service_T s;
 
@@ -482,6 +509,7 @@ void printrunlist() {
   printf(" %-18s = %s\n", "Use process engine", Run.doprocess?"True":"False");
   printf(" %-18s = %d seconds\n", "Poll time", Run.polltime);
   printf(" %-18s = ", "Mail server(s)");
+
   if(Run.mailservers) {
     MailServer_T mta= Run.mailservers;
     for(mta= Run.mailservers; mta; mta= mta->next)
@@ -490,6 +518,7 @@ void printrunlist() {
   } else {
     printf("localhost\n");
   }
+
   printf(" %-18s = %s\n", "Mail from", is_str_defined(Run.MailFormat.from));
   printf(" %-18s = %s\n", "Mail subject",
 	 is_str_defined(Run.MailFormat.subject));
@@ -497,16 +526,15 @@ void printrunlist() {
 	 Run.MailFormat.message?
 	 Run.MailFormat.message:"(not defined)",
 	 Run.MailFormat.message?"..(truncated)":"");
-	 
+
   printf(" %-18s = %s\n", "Start monit httpd", Run.dohttpd?"True":"False");
-
-
   
   if(Run.dohttpd) {
     
     printf(" %-18s = %s\n", "httpd bind address",
 	   Run.bind_addr?Run.bind_addr:"Any/All");
     printf(" %-18s = %d\n", "httpd portnumber", Run.httpdport);
+    printf(" %-18s = %s\n", "httpd signature", Run.httpdsig?"True":"False");
     printf(" %-18s = %s\n", "Use ssl encryption", Run.httpdssl?"True":"False");
 
     if(Run.httpdssl) {
@@ -533,6 +561,45 @@ void printrunlist() {
      
   }
 
+  {
+    Mail_T list;
+    for(list= Run.maillist; list; list= list->next) {
+      printf(" %-18s = %s\n", "Alert mail to", is_str_defined(list->to));
+      printf(" %-18s = ", "Alert on");
+      if(list->events == (~((unsigned int)0))) {
+        printf("All events");
+      } else {
+        if(IS_EVENT_SET(list->events, EVENT_START))
+	  printf("Start ");
+        if(IS_EVENT_SET(list->events, EVENT_STOP))
+	  printf("Stop ");
+        if(IS_EVENT_SET(list->events, EVENT_RESTART))
+	  printf("Restart ");
+        if(IS_EVENT_SET(list->events, EVENT_CHECKSUM))
+	  printf("Checksum ");
+        if(IS_EVENT_SET(list->events, EVENT_RESOURCE))
+	  printf("Resource ");
+        if(IS_EVENT_SET(list->events, EVENT_TIMEOUT))
+	  printf("Timeout ");
+        if(IS_EVENT_SET(list->events, EVENT_TIMESTAMP))
+	  printf("Timestamp ");
+        if(IS_EVENT_SET(list->events, EVENT_SIZE))
+	  printf("Size ");
+        if(IS_EVENT_SET(list->events, EVENT_CONNECTION))
+	  printf("Connection ");
+        if(IS_EVENT_SET(list->events, EVENT_PERMISSION))
+	  printf("Permission ");
+        if(IS_EVENT_SET(list->events, EVENT_UID))
+	  printf("Uid ");
+        if(IS_EVENT_SET(list->events, EVENT_GID))
+	  printf("Gid ");
+        if(IS_EVENT_SET(list->events, EVENT_UNMONITOR))
+	  printf("Unmonitor ");
+      }
+      printf("\n");
+    }
+  }
+
   printf("\n");
   
 }
@@ -583,10 +650,26 @@ void printservice(Service_T s) {
     if(d->dependant != NULL)
       printf(" %-20s = %s\n", "Depends on Service", d->dependant);
 
-  if(s->checksum)
-    printf(" %-20s = if failed %s then %s\n",
-           "Checksum", s->checksum->md5, actionnames[s->checksum->action]);
-
+  if(s->checksum) {
+    switch (s->checksum->type) {
+    case HASH_MD5:
+        printf(" %-20s = if failed MD5(%s) then %s\n",
+               "Checksum", s->checksum->hash,
+               actionnames[s->checksum->action]);
+        break;
+    case HASH_SHA1:
+        printf(" %-20s = if failed SHA1(%s) then %s\n",
+               "Checksum", s->checksum->hash,
+               actionnames[s->checksum->action]);
+        break;
+    default:
+        printf(" %-20s = if failed UNKNOWN(%s) then %s\n",
+               "Checksum", s->checksum->hash,
+               actionnames[s->checksum->action]);
+        break;
+    }
+  }
+  
   if(s->perm)
     printf(" %-20s = if failed %o then %s\n",
            "Permission", s->perm->perm, actionnames[s->perm->action]);
@@ -955,6 +1038,7 @@ char *get_process_uptime(char *pidfile, char *sep) {
  * Compute an uptime string based on the delta time in seconds. The
  * caller must free the returned string.
  * @param delta seconds. 
+ * @param sep string separator
  * @return an uptime string
  */
 char *get_uptime(time_t delta, char *sep) {
@@ -989,12 +1073,27 @@ char *get_uptime(time_t delta, char *sep) {
 
 
 /**
- * @return a md5 checksum for the given file, or NULL if error.
+ * @return a checksum for the given file, or NULL if error.
  */
-char *get_md5sum(char *file) {
+char *get_checksum(char *file, int hashtype) {
+
+  int hashlength=16;
 
   ASSERT(file);
 
+  switch(hashtype) {
+  case HASH_MD5:
+      hashlength=16;
+      break;
+
+  case HASH_SHA1:
+      hashlength=20;
+      break;
+      
+  default:
+      return NULL;
+  }
+
   if(isreg_file(file)) {
     
     FILE *f= fopen(file, "r");
@@ -1002,13 +1101,24 @@ char *get_md5sum(char *file) {
     if(f) {
       
       int i;
-      unsigned char md5buf[16];
+      unsigned char buf[hashlength];
       char result[STRLEN];
       char *r= result;
+      int fresult=0;
 
       *result=0;
+
+      switch(hashtype) {
+      case HASH_MD5:
+          fresult=md5_stream(f, buf);
+          break;
+          
+      case HASH_SHA1:
+          fresult=sha_stream(f, buf);
+          break;
+      }
       
-      if(md5_stream(f, md5buf)) {
+      if(fresult) {
 	
 	fclose(f);
 	
@@ -1018,8 +1128,8 @@ char *get_md5sum(char *file) {
       
       fclose(f);
       
-      for(i= 0; i < 16; ++i)
-	r+= snprintf(r, STRLEN-(r-result) ,"%02x", md5buf[i]);
+      for(i= 0; i < hashlength; ++i)
+	r+= snprintf(r, STRLEN-(r-result) ,"%02x", buf[i]);
     
       return (xstrdup(result));
       
@@ -1033,26 +1143,39 @@ char *get_md5sum(char *file) {
 
 
 /**
- * @param file A file to open and compute a md5 checksum for
+ * @param file A file to open and compute a checksum for
  * @param sum A previous checksum computed for this file
- * @return TRUE if computation of md5sum on the given file, is equal
+ * @return TRUE if computation of hash on the given file, is equal
  * to the given sum, otherwise return FALSE. FALSE is also returned if
- * md5_stream fails.
+ * *hash*_stream fails.
  */
-int check_md5(char *file, char *sum) {
+int check_hash(char *file, char *sum, int hashtype) {
 
   char *newSum;
 
   ASSERT(file);
   ASSERT(sum);
 
-  newSum= get_md5sum(file);
+  newSum= get_checksum(file, hashtype);
+
 
   if(newSum) {
     
     int rv;
     
-    rv= (!strncmp(sum, newSum, 32));
+    switch(hashtype) {
+    case HASH_MD5:
+        rv= (!strncmp(sum, newSum, 32));
+        break;
+        
+    case HASH_SHA1:
+        rv= (!strncmp(sum, newSum, 40));
+        break;
+
+    default:
+        rv= FALSE;
+    }
+    
     FREE(newSum);
     
     return (rv);
@@ -1072,22 +1195,20 @@ int check_md5(char *file, char *sum) {
  */
 char *url_encode(char *uri) {
 
-  static unsigned char hexchars[]= "0123456789ABCDEF";
   register int x, y;
   unsigned char *str;
+  static unsigned char hexchars[]= "0123456789ABCDEF";
 
   ASSERT(uri);
 
   str= (unsigned char *)xmalloc(3 * strlen(uri) + 1);
 
   for(x = 0, y = 0; uri[x]; x++, y++) {
-
     if(is_unsafe((unsigned char*) &uri[x])) {
       str[y++] = '%';
       str[y++] = hexchars[(unsigned char) uri[x] >> 4];
       str[y] = hexchars[(unsigned char) uri[x] & 0xf];
     } else str[y]= (unsigned char)uri[x];
-
   }
 
   str[y] = '\0';
@@ -1097,6 +1218,29 @@ char *url_encode(char *uri) {
 }
 
 
+/**
+ * Unescape an url string. The <code>url</code> parameter is modified
+ * by this method.
+ * @param url an escaped url string
+ * @return A pointer to the unescaped <code>url</code>string
+ */
+char *url_decode(char *url) {
+
+  register int x,y;
+
+  if(!(url&&*url)) return url;
+  replace_char(url, '+', ' ');
+  for(x=0,y=0;url[y];++x,++y) {
+    if((url[x] = url[y]) == '%') {
+      url[x]= x2c(&url[y+1]);
+      y+=2;
+    }
+  }
+  url[x]= 0;
+  return url;
+}
+
+
 /**
  * @return a Basic Authentication Authorization string (RFC 2617),
  * with credentials from the Run object, The string "\r\n" if
@@ -1201,8 +1345,6 @@ void redirect_stdfd() {
 
   int i;
   
-  Run.have_tty= FALSE;
-
   for(i= 0; i < 3; i++)
     if(close(i) == -1 || open("/dev/null", O_RDWR) != i)
       log("Cannot reopen standard file descriptor (%d) -- %s\n", i, STRERROR);
@@ -1268,7 +1410,7 @@ int compare_user_credentials(char *uname, char *outside) {
   switch (c->digesttype) {
   case DIGEST_CLEARTEXT:
   {
-    strcpy(outside_crypt, outside); 
+    strncpy(outside_crypt, outside, STRLEN); 
 
     break;
   }   
@@ -1283,11 +1425,11 @@ int compare_user_credentials(char *uname, char *outside) {
      * the '$' around the id are still part of the id. 
      */
 
-    strcpy(id, c->passwd);
+    strncpy(id, c->passwd, STRLEN);
     temp= strchr(id+1, '$')+1;
     *temp= '\0';
 
-    strcpy(salt, c->passwd+strlen(id));
+    strncpy(salt, c->passwd+strlen(id), STRLEN);
     temp= strchr(salt, '$');
     *temp= '\0';
 
@@ -1306,7 +1448,7 @@ int compare_user_credentials(char *uname, char *outside) {
     char * temp;
     snprintf(salt, 3, "%c%c", c->passwd[0], c->passwd[1]);
     temp= crypt(outside, salt);
-    strcpy(outside_crypt, temp); 
+    strncpy(outside_crypt, temp, STRLEN); 
     
     break;
   }
@@ -1365,3 +1507,19 @@ static int is_unsafe(unsigned char *c) {
 
 }
 
+
+/**
+ * Convert a hex char to a char
+ */
+static char x2c(char *hex) {
+  
+  register char digit;
+  
+  digit = ((hex[0] >= 'A') ? ((hex[0] & 0xdf) - 'A')+10 : (hex[0] - '0'));
+  digit *= 16;
+  digit += (hex[1] >= 'A' ? ((hex[1] & 0xdf) - 'A')+10 : (hex[1] - '0'));
+  
+  return(digit);
+  
+}
+
